@@ -1489,7 +1489,14 @@ class LangServer:
         pool.close()
         pool.join()
         for path, result in results.items():
-            result_obj = result.get()
+            try:
+                result_obj = result.get()
+            except Exception as e:
+                result_obj = (
+                    "An exception has occured while initialising the workspace.\n"
+                    f"Exception({(type(e))}): {e}\n"
+                    + f"Traceback: {traceback.format_exc()}"
+                )
             if isinstance(result_obj, str):
                 self.post_message(
                     f"Initialization failed for file {path}: {result_obj}"
